<?php
/*
 * Classe permettant de se connecter  l'outil de suivi de bugs Mantis (http://mantisbt.org)
 * L'application MantisConnect doit tre installe avec Mantis (www.futureware.biz/mantisconnect/)
 * MantisConnect a t test en version 1.1.0 (Version Subversion)
 * Le module Soap de PHP doit tre install afin de faire appel aux web-services de MantisConnect
 * Un compte doit tre cr sur l'outil Mantis afin de s'authentifier
 * 
 * Un moyen de connatre les possibilits avec MantisConnect :
 * http://www.soapclient.com/SOAPAttachment.html
 * 
 * Aide :
 * http://www.mantisbt.org/bugs/api/soap/mantisconnect.php?wsdl
 * http://www.mantisbt.org/bugs/api/soap/mantisconnect.php#
 */
class TroubleTickets_MantisConnect {
	
	//Variables
	var $url; /* Url d'accs aux web-services de MantisConnect ex: http://mondomaine.com/mc/mantisconnect.php?wsdl */
	var $identifiant; /* Identifiant du compte utilisateur sur Mantis */
	var $motdepasse; /* Mot de passe du compte utilisateur sur Mantis */
	var $decodageUTF8actif; /* Si  vrai, on dcode tous les rsultats du Web-services en UTF8 */

	//Constructeur
	function __construct($url='', $identifiant='', $motdepasse='', $decodageUTF8actif=true) {
		$this->url = $url;
		$this->identifiant = $identifiant;
		$this->motdepasse = $motdepasse;
		$this->decodageUTF8actif = $decodageUTF8actif;
	}
	
	/* Vrifie si le module PHP soap est actif */
	function verifieprerequis() {
		$extensionsphp = get_loaded_extensions();
		for ($i=0;$i<=count($extensionsphp)-1;$i++) {
			if ($extensionsphp[$i] == 'soap') {
				return true;
			}
		}		
		return false;
	}
	
	/* Test une connexion  l'outil */
	function testconnexion() {
		try {
			$connexion = new SoapClient($this->url);
			$version = $connexion->__soapCall('mc_version', array());
		} catch (SoapFault $fault) {
    		bab_debug("SOAP Fault: faultcode: {$fault->faultcode}, faultstring: {$fault->faultstring}");
			return false;
		}
		return true;
	}
	
	/* Retourne les informations d'un bug tel que MantisConnect le renvoit */
	function bug($idbug) {
		$bug = false;
		try {
			$connexion = new SoapClient($this->url);
			$bugexiste = $connexion->__soapCall("mc_issue_exists", array(
				'username' => $this->identifiant,
				'password' => $this->motdepasse,
				'issue' => $idbug
			));
			if ($bugexiste) {
				$bug = $connexion->__soapCall("mc_issue_get", array(
					'username' => $this->identifiant,
					'password' => $this->motdepasse,
					'issue_id' => $idbug
				));
			}
		} catch (SoapFault $fault) {
    		bab_debug("Erreur SOAP: faultcode: {$fault->faultcode}, faultstring: {$fault->faultstring}");
			return false;
		}
		return $bug;
	}
	
	/* Retourne la version de MantisConnect */
	function mantisconnectversion() {
		try {
			$connexion = new SoapClient($this->url);
			$version = $connexion->__soapCall('mc_version', array());
		} catch (SoapFault $fault) {
    		bab_debug("SOAP Fault: faultcode: {$fault->faultcode}, faultstring: {$fault->faultstring}");
			return '';
		}
		if ($this->decodageUTF8actif) {
			$version = utf8_decode($version);
		}
		return $version;
	}
	
}

?>
