<?php
/************************************************************************
 * OVIDENTIA http://www.ovidentia.org                                   *
 ************************************************************************
 * Copyright (c) 2008 by CANTICO ( http://www.cantico.fr )              *
 ************************************************************************/
include_once 'base.php';

//Chargement des fonctions
global $babAddonPhpPath;
global $babInstallPath;
include_once $babAddonPhpPath."fonctionsgen.php";
include_once $babAddonPhpPath."fonctions.php";


/* Gestion des containers OVML : cette fonction place dans le fichier ovml.php suffit au moteur d'Ovidentia
   pour interprter les containers

   Exemples d'utilisation des containers :
	Avoir des informations sur l'utilisateur courant :
	<OCAddon name="troubletickets" action="User">
		<OCIfEqual expr1="<OVUserIsLogged>" expr2="1">
	    	Utilisateur authentifi avec l'ID n<OVUserId>
		</OCIfEqual>
	    <OCIfEqual expr1="<OVUserIsLogged>" expr2="0">
	    	Utilisateur non authentifi
		</OCIfEqual>
	</OCAddon>

	Valide le module MyCanticoAccount :
	<OCAddon name="troubletickets" action="ValidationOfMyCanticoAccount" version="1.0">
		<OCIfEqual expr1="<OVValidation>" expr2="invalidversion">
	    	Version invalide
		</OCIfEqual>
		<OCIfNotEqual expr1="<OVValidation>" expr2="invalidversion">
	    	Version valide
		</OCIfNotEqual>
	</OCAddon>

	Rcupre les informations sur le projet 3 :
	<OCAddon name="troubletickets" action="Project" idproject="3">
	    <OVProjectName> (client : <OVProjectNameCustomer>)
	</OCAddon>

	Rcupre la liste des projets dont l'utilisateur courant est client :
	<OCAddon name="troubletickets" action="Projects" accessright="customer">
	    <OVProjectName> (client : <OVProjectNameCustomer>)
	</OCAddon>
	
	Rcupre les informations sur le ticket 57 :
	<OCAddon name="troubletickets" action="Ticket" idticket="57">
	   Titre : <OVTicketTitle><br />
	   Date : <OVTicketDate><br />
	   Description : <OVTicketDescription><br />
	   Auteur : <OVTicketNameAuthor><br />
	</OCAddon>
	
	Rcupre la liste des tickets en tenant compte des droits d'accs, on tri sur la date en dcroissant et on filtre sur le projet n8
	<OCAddon name="troubletickets" action="Tickets" orderobject="date" ordertype="desc" filterproject="8">
	<div>
		  Titre : (ID : <OVTicketId>) <OVTicketTitle><br />
		  Date : <OVTicketDate><br />
		  Auteur : <OVTicketNameAuthor><br />
		  Projet : <OVTicketNameProject> (<OVTicketIdProject>)<br />
	      <hr />
	</div>
	</OCAddon>

	Cr un ticket en tant que client dans le projet n2 :
	<OCAddon name="troubletickets" action="CreateTicket" idauthor="71" authortype="customer" idproject="2" title="Mon premier ticket cr par OVML" description="Ca marche bien" severity="2">
	  <OCIfEqual expr1="<OVCreateTicketError>" expr2="0">
      	Le ticket a bien t cr. Ticket n<OVCreateTicketId>
      </OCIfEqual>
	  <OCIfEqual expr1="<OVCreateTicketError>" expr2="1">
      	Il manque des paramtres
      </OCIfEqual>
	  <OCIfEqual expr1="<OVCreateTicketError>" expr2="2">
      	Vous n'avez pas les droits d'accs pour crer un ticket dans le projet
      </OCIfEqual>
	</OCAddon>

*/
function troubletickets_ovml($args) {
	$donnees = array();
	
	if (isset($args['action'])) {
		switch($args['action']){
			case 'User':
			case 'user':
				/* Retourne des informations sur l'utilisateur courant afin de savoir s'il est authentifi
				   Peut tre utilis lors de la cration d'un ticket */
				$tab = array();
				if (troubletickets_estauthentifie()) {
					$ficheannuaire = troubletickets_ficheannuaire();
					$tab['UserIsLogged'] = 1;
					$tab['UserId'] = troubletickets_idusercourant();
					$tab['UserLastName'] = $ficheannuaire['sn'];
					$tab['UserFirstName'] = $ficheannuaire['givenname'];
					/* Il arrive que l'adresse mail ne soit pas rcuprable */
					if (isset($ficheannuaire['email'])) {
						$tab['UserEmail'] = $ficheannuaire['email'];
					} else {
						$tab['UserEmail'] = '';
					}					
				} else {
					$tab['UserIsLogged'] = 0;
					$tab['UserId'] = '';
					$tab['UserLastName'] = '';
					$tab['UserFirstName'] = '';
					$tab['UserEmail'] = '';
				}
				$donnees[] = $tab;
				break;
			case 'ValidationOfMyCanticoAccount':
			case 'validationofmycanticoaccount':
				/* Valide la version du module MyCanticoAccount (module spcifique destins aux clients de la socit Cantico) : il
				   est possible que la version actuelle du module TroubleTickets ne soit plus compatible avec le module
				   MyCanticoAccount. Dans ce cas, une nouvelle version du module MyCanticoAccount sortirait et rendrait obsolte
				   l'ancienne version. Ce container permet de vrifier que la version utilise par le client est compatible :
				   retourne la valeur validversion ou invalidversion dans la variable Validation */
				$res = 'invalidversion';
				if (isset($args['version'])) {
					if (is_numeric($args['version'])) {
						if (strnatcmp($args['version'], '1.3') >= 0) {
							$res = 'validversion';
						}
					}
				}
				$donnees[] = array('Validation' => $res);
				break;
			case 'Projects':
			case 'projects':
				/* Retourne les informations sur tous les projets disponibles  l'utilisateur courant. Filtres : projet, droits d'accs */
				$projet = '';
				if (isset($args['idproject'])) {
					if (is_numeric($args['idproject'])) {
						$projet = $args['idproject'];
					}
				}
				$projets = troubletickets_listeprojets($projet); /* ne tient pas compte des droits d'accs */
				if (isset($args['accessright'])) {
					switch($args['accessright']) {
						case 'customer':
							/* On retourne la liste des projets dont l'utilisateur courant est client */
							for ($i=0;$i<=count($projets)-1;$i++) {
								if (troubletickets_adroitsprojets('clients', $projets[$i]->id)) {
									$tab = array();
									$tab['ProjectId'] = $projets[$i]->id;
									$tab['ProjectName'] = $projets[$i]->nom;
									$tab['ProjectIdCustomer'] = $projets[$i]->client;
									$client = $projets[$i]->client();
									$tab['ProjectNameCustomer'] = $client[0]->nom;
									$donnees[] = $tab;
								}
							}
							break;
						case 'speaker':
							/* On retourne la liste des projets dont l'utilisateur courant est intervenant */
							for ($i=0;$i<=count($projets)-1;$i++) {
								if (troubletickets_adroitsprojets('intervenants', $projets[$i]->id)) {
									$tab = array();
									$tab['ProjectId'] = $projets[$i]->id;
									$tab['ProjectName'] = $projets[$i]->nom;
									$tab['ProjectIdCustomer'] = $projets[$i]->client;
									$client = $projets[$i]->client();
									$tab['ProjectNameCustomer'] = $client[0]->nom;
									$donnees[] = $tab;
								}
							}
							break;
						case 'reading':
							/* On retourne la liste des projets dont l'utilisateur courant a des droits de lecture */
							for ($i=0;$i<=count($projets)-1;$i++) {
								if (troubletickets_adroitsprojets('lecturetickets', $projets[$i]->id)) {
									$tab = array();
									$tab['ProjectId'] = $projets[$i]->id;
									$tab['ProjectName'] = $projets[$i]->nom;
									$tab['ProjectIdCustomer'] = $projets[$i]->client;
									$client = $projets[$i]->client();
									$tab['ProjectNameCustomer'] = $client[0]->nom;
									$donnees[] = $tab;
								}
							}
							break;
					}
				} else {
					/* Aucun filtre sur les droits d'accs, on vrifie donc les droits clients, intervenants et lecturetickets */
					for ($i=0;$i<=count($projets)-1;$i++) {
						if (troubletickets_adroitsprojets('clients', $projets[$i]->id) || troubletickets_adroitsprojets('lecturetickets', $projets[$i]->id) || troubletickets_adroitsprojets('intervenants', $projets[$i]->id)) {
							$tab = array();
							$tab['ProjectId'] = $projets[$i]->id;
							$tab['ProjectName'] = $projets[$i]->nom;
							$tab['ProjectIdCustomer'] = $projets[$i]->client;
							$client = $projets[$i]->client();
							$tab['ProjectNameCustomer'] = $client[0]->nom;
							$donnees[] = $tab;
						}
					}
				}
				break;
			case 'Project':
			case 'project':
				/* Retourne les informations sur un projet : l'utilisateur courant doit avoir des droits d'accs clients, lecturetickets ou intervenants */
				if (isset($args['idproject'])) {
					if (is_numeric($args['idproject'])) {
						/* Vrifie les droits d'accs */
						if (troubletickets_adroitsprojets('clients', $args['idproject']) || troubletickets_adroitsprojets('lecturetickets', $args['idproject']) || troubletickets_adroitsprojets('intervenants', $args['idproject'])) {
							$projet = new TroubleTickets_Projet($args['idproject']);
							$projet->charge();
							$tab = array();
							$tab['ProjectId'] = $projet->id;
							$tab['ProjectName'] = $projet->nom;
							$tab['ProjectIdCustomer'] = $projet->client;
							$client = $projet->client();
							$tab['ProjectNameCustomer'] = $client[0]->nom;
							$donnees[] = $tab;
						}
					}
				}				
				break;
			case 'Tickets':
			case 'tickets':
				/* Liste des tickets. Filtres : filterproject, filtercustomer, filterstatute (1 : en attente,2 : en cours,3 : demande d'info,4 : info,5 : demande de cloture,6 : cloture accepte,7 : cloture refuse,8 : ticket non accept,9  :ticket ferm,open), filterseverity(1:mineur,2:bloquant,3:majeur,4:amlioration), filterinterlocutor, filtersearch (chane  rechercher), filtersearchtype (titles, descriptions), filtercurrentpage, filternumberofticketsbypages. Ordres : ordertype(asc ou desc (desc par dfaut)), orderobject(idticket, statute, date, severity) */
				$ordreobjet = '';
				$ordretype = 'desc';
				$filtreprojet = '';
				$filtreclient = '';
				$filtrestatut = '';
				$filtregravite = '';
				$filtrecategoriecloture = '';
				$filtreinterlocuteur = '';
				$filtrerecherche = '';
				$filtrerecherchetype = '';
				$pagecourante = '';
				$nbticketsparpages = '';
				/* Vrifie les paramtres demands */
				if (isset($args['ordertype'])) {
					if ($args['ordertype'] == 'asc') {
						$ordretype = 'asc';
					}
				}
				if (isset($args['orderobject'])) {
					switch($args['orderobject']) {
						case 'idticket':
							$ordreobjet = 'id';
							break;
						case 'statute':
							$ordreobjet = 'statutticket';
							break;
						case 'date':
							$ordreobjet = 'dateticket';
							break;
						case 'severity':
							$ordreobjet = 'gravite';
							break;
					}
				}
				if (isset($args['filterproject'])) {
					if (is_numeric($args['filterproject'])) {
						$filtreprojet = $args['filterproject'];
					}
				}
				if (isset($args['filtercustomer'])) {
					if (is_numeric($args['filtercustomer'])) {
						$filtreclient = $args['filtercustomer'];
					}
				}
				if (isset($args['filterstatute'])) {
					if ($args['filterstatute'] == 'open') {
						$filtrestatut = 'ouvert';
					} else {
						$filtrestatut = $args['filterstatute'];
					}
				}
				if (isset($args['filterseverity'])) {
					if (is_numeric($args['filterseverity'])) {
						$filtreclient = $args['filterseverity'];
					}
				}
				if (isset($args['filterinterlocutor'])) {
					if (is_numeric($args['filterinterlocutor'])) {
						$filtreinterlocuteur = $args['filterinterlocutor'];
					}
				}
				if (isset($args['filtersearch'])) {
					if ($args['filtersearch'] != '') { /* vrifie si la chane n'est pas vide */
						if (isset($args['filtersearchtype'])) {
							if ($args['filtersearchtype'] == 'titles') {
								$filtrerecherchetype = 'titres';
							}
							if ($args['filtersearchtype'] == 'descriptions') {
								$filtrerecherchetype = 'descriptions';
							}
							if ($args['filtersearchtype'] != 'titles' && $args['filtersearchtype'] != 'descriptions') {
								$filtrerecherche = '';
								$filtrerecherchetype = '';
							} else {
								$filtrerecherche = $args['filtersearch'];
							}
						}
					}
				}
				if (isset($args['filtercurrentpage'])) {
					if (is_numeric($args['filtercurrentpage'])) {
						$pagecourante = $args['filtercurrentpage'];
					}
				}
				if (isset($args['filternumberofticketsbypages']) && $pagecourante != '') {
					if (is_numeric($args['filternumberofticketsbypages'])) {
						$nbticketsparpages = $args['filternumberofticketsbypages'];
					}
				}
				/* La fonction troubletickets_listetickets gre dj les droits d'accs */			
				$tickets = troubletickets_listetickets($ordreobjet, $ordretype, $filtreprojet, $filtreclient, $filtrestatut, $filtregravite, $filtreinterlocuteur, $filtrerecherche, $filtrerecherchetype, $pagecourante, $nbticketsparpages, '', '', 'creation', '', '', true, true, $filtrecategoriecloture);
				$nbTickets = count($tickets);
				for ($i=0;$i<=$nbTickets-1;$i++) {
					$tab = array();
					$tab['TicketId'] = $tickets[$i]->id;
					$tab['TicketIdProject'] = $tickets[$i]->projet;
					$projet = troubletickets_listeprojets($tickets[$i]->projet);
					$tab['TicketNameProject'] = $projet[0]->nom;
					$tab['TicketReference'] = $tickets[$i]->reference;
					$tab['TicketIdAuthor'] = $tickets[$i]->auteur;
					$auteur = troubletickets_ficheannuaire($tickets[$i]->auteur);
					$tab['TicketNameAuthor'] = $auteur['givenname'].' '.$auteur['sn'];
					$tab['TicketTitle'] = $tickets[$i]->titre;
					$tab['TicketDescription'] = $tickets[$i]->description;
					$tab['TicketIdStatute'] = $tickets[$i]->statutticket;
					$tab['TicketNameStatute'] = $tickets[$i]->nomstatut();
					$tab['TicketIdSeverity'] = $tickets[$i]->gravite;
					$tab['TicketNameSeverity'] = $tickets[$i]->nomgravite();
					$tab['TicketDateSQL'] = $tickets[$i]->dateticket;
					$tab['TicketDate'] = troubletickets_formatedatetime($tickets[$i]->dateticket);
					$tab['TicketDateYear'] = substr($tickets[$i]->dateticket,0,4);
					$tab['TicketDateMonth'] = substr($tickets[$i]->dateticket,5,2);
					$tab['TicketDateDay'] = substr($tickets[$i]->dateticket,8,2);
					$tab['TicketDateHours'] = substr($tickets[$i]->dateticket,11,2);
					$tab['TicketDateMinutes'] = substr($tickets[$i]->dateticket,14,2);
					$tab['TicketColor'] = $tickets[$i]->couleur();
					$tab['TicketIdLevelSpeakers'] = $ticket[$i]->niveauintervenants;
					$niveau = new TroubleTickets_NiveauIntervenants($ticket[$i]->niveauintervenants);
					$niveau->charge();
					$tab['TicketNameLevelSpeakers'] = $niveau->nom;
					$donnees[] = $tab;
				}
				break;
			case 'Ticket':
			case 'ticket':
				/* Retourne les informations sur un ticket : l'utilisateur courant doit avoir des droits d'accs clients, lecturetickets ou intervenants */
				if (isset($args['idticket'])) {
					if (is_numeric($args['idticket'])) {
						$ticket = new TroubleTickets_Ticket($args['idticket']);
						$ticket->charge();
						/* Vrifie les droits d'accs sur le projet */
						if (troubletickets_adroitsprojets('clients', $ticket->projet) || troubletickets_adroitsprojets('lecturetickets', $ticket->projet) || troubletickets_adroitsprojets('intervenants', $ticket->projet)) {
							$tab = array();
							$tab['TicketId'] = $ticket->id;
							$tab['TicketIdProject'] = $ticket->projet;
							$projet = troubletickets_listeprojets($ticket->projet);
							$tab['TicketNameProject'] = $projet[0]->nom;
							$tab['TicketReference'] = $ticket->reference;
							$tab['TicketIdAuthor'] = $ticket->auteur;
							$auteur = troubletickets_ficheannuaire($ticket->auteur);
							$tab['TicketNameAuthor'] = $auteur['givenname'].' '.$auteur['sn'];
							$tab['TicketTitle'] = $ticket->titre;
							$tab['TicketDescription'] = $ticket->description;
							$tab['TicketIdStatute'] = $ticket->statutticket;
							$tab['TicketNameStatute'] = $ticket->nomstatut();
							$tab['TicketIdSeverity'] = $ticket->gravite;
							$tab['TicketNameSeverity'] = $ticket->nomgravite();
							$tab['TicketDateSQL'] = $ticket->dateticket;
							$tab['TicketDate'] = troubletickets_formatedatetime($ticket->dateticket);
							$tab['TicketDateYear'] = substr($ticket->dateticket,0,4);
							$tab['TicketDateMonth'] = substr($ticket->dateticket,5,2);
							$tab['TicketDateDay'] = substr($ticket->dateticket,8,2);
							$tab['TicketDateHours'] = substr($ticket->dateticket,11,2);
							$tab['TicketDateMinutes'] = substr($ticket->dateticket,14,2);
							$tab['TicketColor'] = $ticket->couleur();
							$tab['TicketIdLevelSpeakers'] = $ticket->niveauintervenants;
							$niveau = new TroubleTickets_NiveauIntervenants($ticket->niveauintervenants);
							$niveau->charge();
							$tab['TicketNameLevelSpeakers'] = $niveau->nom;
							$donnees[] = $tab;
						}
					}
				}
				break;
			case 'CreateTicket':
			case 'createticket':
				/* Permet de crer un nouveau ticket
			        Paramtres obligatoires :
					idproject
					idauthor (id de l'utilisateur courant si non spcifi)
					title
					description : tant donn que la description doit tre place
					severity (1 : Minor, 2 : Blocking, 3 : Engrave, 4 : Improvement)
					authortype (customer, speaker)

					Le container renvoie l'id du ticket cr : <OVCreateTicketId>
					Il renvoit aussi un numro d'erreur en cas d'erreur (0 : aucune erreur ; ticket cr, 1 : manque des donnes, 2 : manque droits d'accs)
			     */
				$idticket = '';
				$erreur = 0;
				/* Vrifie les paramtres envoys */
				if (!isset($args['idauthor'])) {
					$args['idauthor'] = troubletickets_idusercourant();
				} else {
					if ($args['idauthor'] == '') {
						$args['idauthor'] = troubletickets_idusercourant();
					}
				}
				if (isset($args['idproject']) && isset($args['idauthor']) && isset($args['title']) && isset($args['description']) && isset($args['severity']) && isset($args['authortype'])) {
					/* Vrifie les droits de cration d'un ticket dans le projet */
					if (is_numeric($args['idproject'])) {
						$projet = new TroubleTickets_Projet($args['idproject']);
						$projet->charge();
						$droitsok = false;
						if ($args['authortype'] == 'customer' && $projet->alesdroits('clients')) {
							$droitsok = true;
						}
						if ($args['authortype'] == 'speaker' && $projet->alesdroits('intervenants')) {
							$droitsok = true;
						}
						if (!$droitsok) {
							$erreur = 2;
						}
						
						/* Vrifie si l'auteur est bien client dans le projet */
						$interlocuteurs = $projet->contactsclients();
						$trouve = false;
						for($i=0;$i<=count($interlocuteurs)-1;$i++) {
							if ($interlocuteurs[$i]->idutilisateur == $args['idauthor']) {
								$trouve = true;
								$droitsok = true;
							}
						}
						if (!$trouve) {
							$erreur = 2;
						}
					}
					if ($args['authortype'] != 'customer' && $args['authortype'] != 'speaker') {
						$erreur = 1;
					}
					/* Vrifie le paramtre severity */
					if ($args['severity'] != 1 && $args['severity'] != 2 && $args['severity'] != 3 && $args['severity'] != 4) {
						$erreur = 1;
					}					
					if ($erreur == 0) {
						$ticket = new TroubleTickets_Ticket();
						$ticket->auteur = $args['idauthor'];
						$ticket->projet = $args['idproject'];
						$ticket->titre = $args['title'];
						$ticket->description = $args['description'];
						$ticket->gravite = $args['severity'];
						if ($args['authortype'] == 'customer') {
							$ticket->ticketcreeparintervenant = false;
						} else {
							$ticket->ticketcreeparintervenant = true;
						}
						$ticket->enregistre();
						$idticket = $ticket->id;
						$ticket->mail();
					}
				} else {
					$erreur = 1;
				}
				/* Rsultat */
				$tab = array();
				$tab['CreateTicketError'] = $erreur;
				$tab['CreateTicketId'] = $idticket;
				$donnees[] = $tab;
				break;
		}
	}
	
	return $donnees;
}

?>
